#include <stdlib.h>
#include <string.h>

#include "../unity/unity.h"
#include "./symcrypt_lib.h"

void setUp(void) {}
void tearDown(void) {}

void test_encrypt(void) {
  char secret[10] = "foo";
  char message[10] = "bar";
  char encrypted[10];

  encrypt(message, secret, encrypted);

  TEST_ASSERT_EQUAL_CHAR('I', encrypted[0]);
  TEST_ASSERT_EQUAL_CHAR('Q', encrypted[1]);
  TEST_ASSERT_EQUAL_CHAR('b', encrypted[2]);
  TEST_ASSERT_EQUAL_CHAR(0, encrypted[3]);
}

void test_decrypt(void) {
  char secret[10] = "foo";
  char message[10] = "IQb";
  char decrypted[10];

  decrypt(message, secret, decrypted);

  TEST_ASSERT_EQUAL_CHAR('b', decrypted[0]);
  TEST_ASSERT_EQUAL_CHAR('a', decrypted[1]);
  TEST_ASSERT_EQUAL_CHAR('r', decrypted[2]);
  TEST_ASSERT_EQUAL_CHAR(0, decrypted[3]);
}

size_t rand_str(char s[], size_t buf_size) {
  size_t len = (rand() % (buf_size - 1)) + 1;
  for (size_t j = 0; j < len; ++j) {
    s[j] = ASCII_BEGIN + (rand() % ASCII_SIZE);
  }
  s[len] = 0;
  return len;
}

void test_encrypt_decrypt_inverses(void) {
  char secret[10];
  char message[10];
  char encrypted[10];
  char decrypted[10];

  for (size_t i = 0; i < 100; ++i) {
    size_t secret_len = rand_str(secret, 10);
    size_t message_len = rand_str(message, 10);

    /* printf("Secret:    %s\n", secret); */
    /* printf("Message:   %s\n", message); */
    encrypt(message, secret, encrypted);
    /* printf("Encrypted: %s\n", encrypted); */
    decrypt(encrypted, secret, decrypted);
    /* printf("Decrypted: %s\n\n", decrypted); */

    TEST_ASSERT_EQUAL_STRING(message, decrypted);
  }
}

int main(void) {
  UNITY_BEGIN();
  RUN_TEST(test_encrypt);
  RUN_TEST(test_decrypt);
  RUN_TEST(test_encrypt_decrypt_inverses);
  return UNITY_END();
}
